//
// servos.c
//
// servo motor hello-world
//
// set lfuse to 0x7E for 20 MHz xtal
//
// Neil Gershenfeld
// 4/8/12
//
// (c) Massachusetts Institute of Technology 2012
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//
// Changes added by Linda Wanders
// FabAcademy2014 - final project development 
// 4/28/14
// 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define position_delay() _delay_ms(1000)

#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_out (1 << PA5)
#define serial_pin_in (1 << PA4)
#define serial_pins PINA

#define servo_port PORTA
#define servo1 (1 << PA6)
#define servo2 (1 << PA7)
#define servo_direction DDRA

#define servoMIN 1250
#define servoMAX 6400
#define servoMID ((servoMAX - servoMIN) / 2 + servoMIN)
#define servoSMALLSTEP 100

int16_t adc0_avg1, adc0_avg2, adc0_avg3; //declare varialble
int16_t adc1_avg1, adc1_avg2, adc1_avg3;

volatile int clock;

char chr1_high, chr2_high, chr3_high, chr4_high, chr5_high, chr6_high;
char chr1_low, chr2_low, chr3_low, chr4_low, chr5_low, chr6_low;

ISR(TIM1_OVF_vect) {
   //
   // interrupt routine for overflow van timer 1 (beginpunt periode)
   //
	set(servo_port, servo1 | servo2);
	
	clock ++; // hoog de clock op met 1 
}

ISR(TIM1_COMPA_vect) {
   //
   // interrupt routine for compare register A - timer 1 (uitzetten van pin)
   //
	clear(servo_port, servo1);
}

ISR(TIM1_COMPB_vect) {
   //
   // interrupt routine for compare register B - timer 1 (uitzetten van pin)
   //
   	clear(servo_port, servo2);
}
void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }



int main(void) {
   //
   // main
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // set up timer 1
   //
   TCCR1A = (0 << COM1A1) | (0 << COM1A0); // Normal port operation, OC1A/OC1B disconnected (doet nu niets automatisch)
   TCCR1B = (0 << CS12) | (1 << CS11) | (0 << CS10) | (1 << WGM13); // prescaler /8, phase and frequency correct PWM, ICR1 TOP
   ICR1 = 25000; // 20 ms frequency (duuratie periode PWM)
   TIMSK1 = (1 << OCIE1B) | (1 << OCIE1A) | (1 << TOIE1); // Output Compare A Match/B Match/Overflow Interrupt Enable
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   set(serial_port, serial_pin_in);
   output(serial_direction, serial_pin_out);
   output(servo_direction, servo1 | servo2);    // set servo pin to output
   //
   //
   // set enable interupts (globaal interrupts aan of uit)
   //
   sei();
   //
   // send servo to starting position
   //
   OCR1A = servoMID;
   OCR1B = servoMID;
   _delay_ms(1000);
   //
   // main loop
   //
   while (1) {
		
	  if (clock >= 3) {
		
		clock=0;
		
		put_char (&serial_port, serial_pin_out, '1'); // call to ID (touchsensor)
		
		
		get_char (&serial_pins, serial_pin_in, &chr1_high); //receive answer
		get_char (&serial_pins, serial_pin_in, &chr1_low);
		get_char (&serial_pins, serial_pin_in, &chr2_high);
		get_char (&serial_pins, serial_pin_in, &chr2_low);
		get_char (&serial_pins, serial_pin_in, &chr3_high);
		get_char (&serial_pins, serial_pin_in, &chr3_low);
		get_char (&serial_pins, serial_pin_in, &chr4_high);
		get_char (&serial_pins, serial_pin_in, &chr4_low);
		get_char (&serial_pins, serial_pin_in, &chr5_high);
		get_char (&serial_pins, serial_pin_in, &chr5_low);
		get_char (&serial_pins, serial_pin_in, &chr6_high);
		get_char (&serial_pins, serial_pin_in, &chr6_low);
		
	 
		adc0_avg1 = (int16_t)(chr1_high << 8); //rebuild values (highbyte/lowbyte)
		adc0_avg1 += (int16_t)(chr1_low);
		adc0_avg2 = (int16_t)(chr2_high << 8); 
		adc0_avg2 += (int16_t)(chr2_low);
		adc0_avg3 = (int16_t)(chr3_high << 8); 
		adc0_avg3 += (int16_t)(chr3_low);
		adc1_avg1 = (int16_t)(chr4_high << 8); 
		adc1_avg1 += (int16_t)(chr4_low);
		adc1_avg2 = (int16_t)(chr5_high << 8); 
		adc1_avg2 += (int16_t)(chr5_low);
		adc1_avg3 = (int16_t)(chr6_high << 8); 
		adc1_avg3 += (int16_t)(chr6_low);
		
		if (adc0_avg1 <= 400) { // when yellow sensor (intv 1) is touched directly, servo's move the tail to the left
			if (OCR1A + servoSMALLSTEP <= servoMAX) {
				OCR1A += servoSMALLSTEP;
			}
		}
		
		/*
		if ((adc0_avg1 >= 700, adc1_avg1 >= 600) >= 3 sec) { // if nothing hapens in 3 seconds time, servo's will move the tail back to centre
				OCR1A = servoMID;
				OCR1B = servoMID;
			}
		}
		*/
		
		if (adc1_avg1 <= 470) { // when green sensor touched directly, servo's move tail right
			if (OCR1A - servoSMALLSTEP >= servoMIN) {
				OCR1A -= servoSMALLSTEP;
			}
		}
		
		
		
		}
		
	    
	
	
	
	
	

	/*
	  OCR1A = servoMID;
      OCR1B = servoMID;
      position_delay();
		
	  OCR1A = servoMID + 1000;
      OCR1B = servoMID + 1000;
      position_delay();
	  
	  OCR1A = servoMID;
      OCR1B = servoMID;
      position_delay();
	  
	  OCR1A = servoMID - 1000;
      OCR1B = servoMID - 1000;                                       
      position_delay();
	 */
 
      }
   }
